import 'dart:async';
import 'package:circular_splash_transition/route.dart';
import 'package:flutter/material.dart';


class CircularSplashController {
  final Duration duration;
  final Color color;
  bool hasCancel = false;
  bool hasRunning = false;

  CircularSplashController(
      {this.duration = const Duration(milliseconds: 300),
        this.color = Colors.white})
      : assert(color != null),
        assert(duration != null);

  void dispose() {
    _pushAndPopController?.close();
  }

  final StreamController<bool> _pushAndPopController = StreamController<bool>();

  Stream<bool> get pushAndPopOut => _pushAndPopController.stream;

  Sink<bool> get pushAndPopIn => _pushAndPopController.sink;

  Future<Object> _runPush(Function runPush) async {
    if (!hasRunning) {
      hasRunning = true;
      hasCancel = false;
      pushAndPopIn.add(true);
      await Future.delayed(Duration(milliseconds: duration.inMilliseconds + 1));
      if (hasCancel == true) {
        hasRunning = false;
        return null;
      }
      hasRunning = false;
      return runPush();
    } else {
      return null;
    }
  }

  void _runPop() async {
    await Future.delayed(Duration(milliseconds: duration.inMilliseconds + 1));
    pushAndPopIn.add(false);
  }

  Future<Object> pushNamed(BuildContext context, String routeName) {
    return _runPush(() {
      return Navigator.pushNamed(context, routeName).then((value) {
        _runPop();
        return value;
      });
    });
  }

  Future<Object> push(BuildContext context, Widget builder) {
    return _runPush(() {
      return Navigator.push(
          context,
          CircularSplashRoute(
              builder: builder, duration: duration, color: color))
          .then((value) {
        _runPop();
        return value;
      });
    });
  }

  void pushReplacement(BuildContext context, Widget builder) {
    _runPush(() {
      Navigator.pushReplacement(
          context,
          CircularSplashRoute(
              builder: builder, duration: duration, color: color));
      dispose();
    });
  }

  void pushReplacementNamed(BuildContext context, String routeName) {
    _runPush(() {
      Navigator.pushReplacementNamed(context, routeName);
      dispose();
    });
  }
}
