import 'package:circular_splash_transition/controller.dart';
import 'package:flutter/material.dart';
import 'dart:async';


// ignore: must_be_immutable
class CircularSplash extends StatefulWidget {
  CircularSplash(
      {Key key,
        @required this.controller,
        @required this.child,
        this.onWillPop})
      : assert(controller != null),
        assert(child != null),
        duration = controller.duration,
        color = controller.color,
        super(key: key) {
    if (onWillPop == null) {
      onWillPop = () => Future.value(true);
    }
  }

  final Duration duration;
  final Color color;
  final CircularSplashController controller;
  final Widget child;
  WillPopCallback onWillPop;

  @override
  _CircularSplashState createState() =>
      _CircularSplashState(controller, duration, color, onWillPop);
}

class _CircularSplashState extends State<CircularSplash>
    with SingleTickerProviderStateMixin {
  _CircularSplashState(
      this._controller, this._duration, this._color, this._onWillPop);

  final CircularSplashController _controller;
  final Duration _duration;
  final Color _color;
  final WillPopCallback _onWillPop;
  Animation<double> _animationScale;
  AnimationController _animationController;

  @override
  void initState() {
    super.initState();
    _animationController =
        AnimationController(vsync: this, duration: _duration);
    _animationScale =
        Tween<double>(begin: 0.0, end: 1.0).animate(_animationController);
  }

  @override
  void dispose() {
    _animationController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () {
        if (_animationController.value > 0) {
          _controller.hasCancel = true;
          _controller.pushAndPopIn.add(false);
          return Future.value(false);
        } else {
          return _onWillPop();
        }
      },
      child: Stack(
        children: <Widget>[
          widget.child,
          StreamBuilder<bool>(
            stream: _controller.pushAndPopOut,
            initialData: false,
            builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
              if (snapshot.data) {
                _animationController.forward();
              }
              if (!snapshot.data) {
                if (_animationController.value > 0) {
                  _animationController.reverse();
                }
              }
              return _pushAndPop();
            },
          ),
        ],
      ),
    );
  }

  Widget _pushAndPop() {
    return AnimatedBuilder(
        animation: _animationScale,
        builder: (context, child) {
          return Center(
            child: Transform.scale(
              scale: _animationScale.value * 2.5,
              child: Container(
                height: MediaQuery.of(context).size.height,
                width: MediaQuery.of(context).size.height,
                decoration:
                BoxDecoration(shape: BoxShape.circle, color: _color),
              ),
            ),
          );
        });
  }
}
